@@ -71,6 +71,10 @@ for CC in ["gcc", "clang"]:
             "CC": CC,
             "FLAGS": [
                 "USE_ZLIB=1",
+                "USE_OT=1",
+                "OT_INC=${HOME}/opt-ot/include",
+                "OT_LIB=${HOME}/opt-ot/lib",
+                "OT_RUNPATH=1",
                 "USE_PCRE=1",
                 "USE_PCRE_JIT=1",
                 "USE_LUA=1",
@@ -142,6 +146,10 @@ matrix.append(
         "FLAGS": get_asan_flags(CC)
         + [
             "USE_ZLIB=1",
+            "USE_OT=1",
+            "OT_INC=${HOME}/opt-ot/include",
+            "OT_LIB=${HOME}/opt-ot/lib",
+            "OT_RUNPATH=1",
             "USE_PCRE=1",
             "USE_PCRE_JIT=1",
             "USE_LUA=1",
