@@ -49,7 +49,7 @@ static inline enum acl_test_res pat2acl(struct pattern *pat)
  */
 void acl_register_keywords(struct acl_kw_list *kwl)
 {
-	LIST_ADDQ(&acl_keywords.list, &kwl->list);
+	LIST_APPEND(&acl_keywords.list, &kwl->list);
 }
 
 /*
@@ -57,7 +57,7 @@ void acl_register_keywords(struct acl_kw_list *kwl)
  */
 void acl_unregister_keywords(struct acl_kw_list *kwl)
 {
-	LIST_DEL(&kwl->list);
+	LIST_DELETE(&kwl->list);
 	LIST_INIT(&kwl->list);
 }
 
@@ -101,6 +101,7 @@ struct acl_keyword *find_acl_kw(const char *kw)
 static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 {
 	struct arg *arg;
+	int unresolved = 0;
 
 	pattern_prune(&expr->pat);
 
@@ -109,6 +110,7 @@ static struct acl_expr *prune_acl_expr(struct acl_expr *expr)
 			break;
 		if (arg->type == ARGT_STR || arg->unresolved) {
 			chunk_destroy(&arg->data.str);
+			unresolved |= arg->unresolved;
 			arg->unresolved = 0;
 		}
 	}
@@ -279,7 +281,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			if (!conv_expr)
 				goto out_free_smp;
 
-			LIST_ADDQ(&(smp->conv_exprs), &(conv_expr->list));
+			LIST_APPEND(&(smp->conv_exprs), &(conv_expr->list));
 			conv_expr->conv = conv;
 			acl_conv_found = 1;
 
@@ -308,8 +310,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 				goto out_free_smp;
 			}
 		}
-		free(ckw);
-		ckw = NULL;
+		ha_free(&ckw);
 	}
 	else {
 		/* This is not an ACL keyword, so we hope this is a sample fetch
@@ -343,7 +344,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 		expr->pat.parse  = aclkw->parse  ? aclkw->parse  : pat_parse_fcts[aclkw->match_type];
 		expr->pat.index  = aclkw->index  ? aclkw->index  : pat_index_fcts[aclkw->match_type];
 		expr->pat.match  = aclkw->match  ? aclkw->match  : pat_match_fcts[aclkw->match_type];
-		expr->pat.delete = aclkw->delete ? aclkw->delete : pat_delete_fcts[aclkw->match_type];
 		expr->pat.prune  = aclkw->prune  ? aclkw->prune  : pat_prune_fcts[aclkw->match_type];
 	}
 
@@ -357,7 +357,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			expr->pat.parse = pat_parse_fcts[PAT_MATCH_BOOL];
 			expr->pat.index = pat_index_fcts[PAT_MATCH_BOOL];
 			expr->pat.match = pat_match_fcts[PAT_MATCH_BOOL];
-			expr->pat.delete = pat_delete_fcts[PAT_MATCH_BOOL];
 			expr->pat.prune = pat_prune_fcts[PAT_MATCH_BOOL];
 			expr->pat.expect_type = pat_match_types[PAT_MATCH_BOOL];
 			break;
@@ -365,7 +364,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			expr->pat.parse = pat_parse_fcts[PAT_MATCH_INT];
 			expr->pat.index = pat_index_fcts[PAT_MATCH_INT];
 			expr->pat.match = pat_match_fcts[PAT_MATCH_INT];
-			expr->pat.delete = pat_delete_fcts[PAT_MATCH_INT];
 			expr->pat.prune = pat_prune_fcts[PAT_MATCH_INT];
 			expr->pat.expect_type = pat_match_types[PAT_MATCH_INT];
 			break;
@@ -375,7 +373,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			expr->pat.parse = pat_parse_fcts[PAT_MATCH_IP];
 			expr->pat.index = pat_index_fcts[PAT_MATCH_IP];
 			expr->pat.match = pat_match_fcts[PAT_MATCH_IP];
-			expr->pat.delete = pat_delete_fcts[PAT_MATCH_IP];
 			expr->pat.prune = pat_prune_fcts[PAT_MATCH_IP];
 			expr->pat.expect_type = pat_match_types[PAT_MATCH_IP];
 			break;
@@ -383,7 +380,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			expr->pat.parse = pat_parse_fcts[PAT_MATCH_STR];
 			expr->pat.index = pat_index_fcts[PAT_MATCH_STR];
 			expr->pat.match = pat_match_fcts[PAT_MATCH_STR];
-			expr->pat.delete = pat_delete_fcts[PAT_MATCH_STR];
 			expr->pat.prune = pat_prune_fcts[PAT_MATCH_STR];
 			expr->pat.expect_type = pat_match_types[PAT_MATCH_STR];
 			break;
@@ -468,7 +464,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			expr->pat.parse = pat_parse_fcts[idx];
 			expr->pat.index = pat_index_fcts[idx];
 			expr->pat.match = pat_match_fcts[idx];
-			expr->pat.delete = pat_delete_fcts[idx];
 			expr->pat.prune = pat_prune_fcts[idx];
 			expr->pat.expect_type = pat_match_types[idx];
 			args++;
@@ -663,7 +658,7 @@ struct acl *prune_acl(struct acl *acl) {
 	free(acl->name);
 
 	list_for_each_entry_safe(expr, exprb, &acl->expr, list) {
-		LIST_DEL(&expr->list);
+		LIST_DELETE(&expr->list);
 		prune_acl_expr(expr);
 		free(expr);
 	}
@@ -731,7 +726,7 @@ struct acl *parse_acl(const char **args, struct list *known_acl, char **err, str
 		}
 
 		LIST_INIT(&cur_acl->expr);
-		LIST_ADDQ(known_acl, &cur_acl->list);
+		LIST_APPEND(known_acl, &cur_acl->list);
 		cur_acl->name = name;
 	}
 
@@ -741,7 +736,7 @@ struct acl *parse_acl(const char **args, struct list *known_acl, char **err, str
 	 */
 	cur_acl->use |= acl_expr->smp->fetch->use;
 	cur_acl->val |= acl_expr->smp->fetch->val;
-	LIST_ADDQ(&cur_acl->expr, &acl_expr->list);
+	LIST_APPEND(&cur_acl->expr, &acl_expr->list);
 	return cur_acl;
 
  out_free_name:
@@ -765,6 +760,7 @@ const struct {
 	{ .name = "HTTP",           .expr = {"req.proto_http",""}},
 	{ .name = "HTTP_1.0",       .expr = {"req.ver","1.0",""}},
 	{ .name = "HTTP_1.1",       .expr = {"req.ver","1.1",""}},
+	{ .name = "HTTP_2.0",       .expr = {"req.ver","2.0",""}},
 	{ .name = "METH_CONNECT",   .expr = {"method","CONNECT",""}},
 	{ .name = "METH_DELETE",    .expr = {"method","DELETE",""}},
 	{ .name = "METH_GET",       .expr = {"method","GET","HEAD",""}},
@@ -834,9 +830,9 @@ static struct acl *find_acl_default(const char *acl_name, struct list *known_acl
 	cur_acl->use |= acl_expr->smp->fetch->use;
 	cur_acl->val |= acl_expr->smp->fetch->val;
 	LIST_INIT(&cur_acl->expr);
-	LIST_ADDQ(&cur_acl->expr, &acl_expr->list);
+	LIST_APPEND(&cur_acl->expr, &acl_expr->list);
 	if (known_acl)
-		LIST_ADDQ(known_acl, &cur_acl->list);
+		LIST_APPEND(known_acl, &cur_acl->list);
 
 	return cur_acl;
 
@@ -1001,9 +997,9 @@ struct acl_cond *parse_acl_cond(const char **args, struct list *known_acl,
 				goto out_free_term;
 			}
 			LIST_INIT(&cur_suite->terms);
-			LIST_ADDQ(&cond->suites, &cur_suite->list);
+			LIST_APPEND(&cond->suites, &cur_suite->list);
 		}
-		LIST_ADDQ(&cur_suite->terms, &cur_term->list);
+		LIST_APPEND(&cur_suite->terms, &cur_term->list);
 		neg = 0;
 	}
 
@@ -1037,11 +1033,11 @@ struct acl_cond *build_acl_cond(const char *file, int line, struct list *known_a
 	if (err)
 		*err = NULL;
 
-	if (!strcmp(*args, "if")) {
+	if (strcmp(*args, "if") == 0) {
 		pol = ACL_COND_IF;
 		args++;
 	}
-	else if (!strcmp(*args, "unless")) {
+	else if (strcmp(*args, "unless") == 0) {
 		pol = ACL_COND_UNLESS;
 		args++;
 	}
@@ -1240,7 +1236,7 @@ int acl_find_targets(struct proxy *p)
 
 	list_for_each_entry(acl, &p->acl, list) {
 		list_for_each_entry(expr, &acl->expr, list) {
-			if (!strcmp(expr->kw, "http_auth_group")) {
+			if (strcmp(expr->kw, "http_auth_group") == 0) {
 				/* Note: the ARGT_USR argument may only have been resolved earlier
 				 * by smp_resolve_args().
 				 */
@@ -1314,6 +1310,26 @@ int init_acl()
 	return err;
 }
 
+void free_acl_cond(struct acl_cond *cond)
+{
+	struct acl_term_suite *suite, *suiteb;
+	struct acl_term *term, *termb;
+
+	if (!cond)
+		return;
+
+	list_for_each_entry_safe(suite, suiteb, &cond->suites, list) {
+		list_for_each_entry_safe(term, termb, &suite->terms, list) {
+			LIST_DELETE(&term->list);
+			free(term);
+		}
+		LIST_DELETE(&suite->list);
+		free(suite);
+	}
+
+	free(cond);
+}
+
 /************************************************************************/
 /*      All supported sample and ACL keywords must be declared here.    */
 /************************************************************************/
