@@ -43,7 +43,7 @@
 
 
 static struct list pid_list = LIST_HEAD_INIT(pid_list);
-static struct pool_head *pool_head_pid_list;
+static struct pool_head *pool_head_pid_list __read_mostly;
 __decl_spinlock(pid_list_lock);
 
 struct extcheck_env {
@@ -124,7 +124,7 @@ static struct pid_list *pid_list_add(pid_t pid, struct task *t)
 	LIST_INIT(&elem->list);
 
 	HA_SPIN_LOCK(PID_LIST_LOCK, &pid_list_lock);
-	LIST_ADD(&pid_list, &elem->list);
+	LIST_INSERT(&pid_list, &elem->list);
 	HA_SPIN_UNLOCK(PID_LIST_LOCK, &pid_list_lock);
 
 	return elem;
@@ -138,7 +138,7 @@ static void pid_list_del(struct pid_list *elem)
 		return;
 
 	HA_SPIN_LOCK(PID_LIST_LOCK, &pid_list_lock);
-	LIST_DEL(&elem->list);
+	LIST_DELETE(&elem->list);
 	HA_SPIN_UNLOCK(PID_LIST_LOCK, &pid_list_lock);
 
 	if (!elem->exited)
@@ -358,15 +358,13 @@ err:
 	if (check->envp) {
 		for (i = 0; i < EXTCHK_SIZE; i++)
 			free(check->envp[i]);
-		free(check->envp);
-		check->envp = NULL;
+		ha_free(&check->envp);
 	}
 
 	if (check->argv) {
 		for (i = 1; i < 5; i++)
 			free(check->argv[i]);
-		free(check->argv);
-		check->argv = NULL;
+		ha_free(&check->argv);
 	}
 	return 0;
 }
@@ -604,7 +602,7 @@ struct task *process_chk_proc(struct task *t, void *context, unsigned int state)
 
 /* Parses the "external-check" proxy keyword */
 int proxy_parse_extcheck(char **args, int section, struct proxy *curpx,
-                         struct proxy *defpx, const char *file, int line,
+                         const struct proxy *defpx, const char *file, int line,
                          char **errmsg)
 {
 	int cur_arg, ret = 0;
@@ -648,7 +646,7 @@ error:
 	return -1;
 }
 
-int proxy_parse_external_check_opt(char **args, int cur_arg, struct proxy *curpx, struct proxy *defpx,
+int proxy_parse_external_check_opt(char **args, int cur_arg, struct proxy *curpx, const struct proxy *defpx,
 				   const char *file, int line)
 {
 	int err_code = 0;
