@@ -62,7 +62,7 @@
  *     |    +----------+
  *     |       |  ^
  *     |  may  |  |
- *     |       v  | EAGAIN (cant)
+ *     |       v  | EAGAIN (can't)
  *     |     +--------+
  *     |     | READY  |     (READY=1, ACTIVE=1)
  *     |     +--------+
@@ -91,18 +91,19 @@
 #include <haproxy/cfgparse.h>
 #include <haproxy/fd.h>
 #include <haproxy/global.h>
+#include <haproxy/log.h>
 #include <haproxy/port_range.h>
 #include <haproxy/tools.h>
 
 
-struct fdtab *fdtab = NULL;     /* array of all the file descriptors */
-struct polled_mask *polled_mask = NULL; /* Array for the polled_mask of each fd */
-struct fdinfo *fdinfo = NULL;   /* less-often used infos for file descriptors */
+struct fdtab *fdtab             __read_mostly = NULL;  /* array of all the file descriptors */
+struct polled_mask *polled_mask __read_mostly = NULL;  /* Array for the polled_mask of each fd */
+struct fdinfo *fdinfo           __read_mostly = NULL;  /* less-often used infos for file descriptors */
 int totalconn;                  /* total # of terminated sessions */
 int actconn;                    /* # of active sessions */
 
-struct poller pollers[MAX_POLLERS];
-struct poller cur_poller;
+struct poller pollers[MAX_POLLERS] __read_mostly;
+struct poller cur_poller           __read_mostly;
 int nbpollers = 0;
 
 volatile struct fdlist update_list; // Global update list
@@ -110,7 +111,7 @@ volatile struct fdlist update_list; // Global update list
 THREAD_LOCAL int *fd_updt  = NULL;  // FD updates list
 THREAD_LOCAL int  fd_nbupdt = 0;   // number of updates in the list
 THREAD_LOCAL int poller_rd_pipe = -1; // Pipe to wake the thread
-int poller_wr_pipe[MAX_THREADS]; // Pipe to wake the threads
+int poller_wr_pipe[MAX_THREADS] __read_mostly; // Pipe to wake the threads
 
 volatile int ha_used_fds = 0; // Number of FD we're currently using
 static struct fdtab *fdtab_addr;  /* address of the allocated area containing fdtab */
@@ -166,7 +167,7 @@ redo_last:
 	}
 	__ha_barrier_store();
 	/* since we're alone at the end of the list and still locked(-2),
-	 * we know noone tried to add past us. Mark the end of list.
+	 * we know no one tried to add past us. Mark the end of list.
 	 */
 	_GET_PREV(fd, off) = last;
 	_GET_NEXT(fd, off) = -1;
@@ -303,13 +304,15 @@ done:
  */
 void _fd_delete_orphan(int fd)
 {
-	if (fdtab[fd].linger_risk) {
+	if (fdtab[fd].state & FD_LINGER_RISK) {
 		/* this is generally set when connecting to servers */
 		DISGUISE(setsockopt(fd, SOL_SOCKET, SO_LINGER,
 			   (struct linger *) &nolinger, sizeof(struct linger)));
 	}
 	if (cur_poller.clo)
 		cur_poller.clo(fd);
+
+	port_range_release_port(fdinfo[fd].port_range, fdinfo[fd].local_port);
 	polled_mask[fd].poll_recv = polled_mask[fd].poll_send = 0;
 
 	fdtab[fd].state = 0;
@@ -317,15 +320,13 @@ void _fd_delete_orphan(int fd)
 #ifdef DEBUG_FD
 	fdtab[fd].event_count = 0;
 #endif
-	port_range_release_port(fdinfo[fd].port_range, fdinfo[fd].local_port);
 	fdinfo[fd].port_range = NULL;
 	fdtab[fd].owner = NULL;
-	fdtab[fd].exported = 0;
 	/* perform the close() call last as it's what unlocks the instant reuse
 	 * of this FD by any other thread.
 	 */
 	close(fd);
-	_HA_ATOMIC_SUB(&ha_used_fds, 1);
+	_HA_ATOMIC_DEC(&ha_used_fds);
 }
 
 #ifndef HA_HAVE_CAS_DW
@@ -338,7 +339,7 @@ __decl_thread(__decl_rwlock(fd_mig_lock));
 void fd_delete(int fd)
 {
 	/* we must postpone removal of an FD that may currently be in use
-	 * by another thread. This can happend in the following two situations:
+	 * by another thread. This can happen in the following two situations:
 	 *   - after a takeover, the owning thread closes the connection but
 	 *     the previous one just woke up from the poller and entered
 	 *     the FD handler iocb. That thread holds an entry in running_mask
@@ -375,7 +376,7 @@ int fd_takeover(int fd, void *expected_owner)
 	int ret = -1;
 
 #ifndef HA_HAVE_CAS_DW
-	if (_HA_ATOMIC_OR(&fdtab[fd].running_mask, tid_bit) == tid_bit) {
+	if (_HA_ATOMIC_OR_FETCH(&fdtab[fd].running_mask, tid_bit) == tid_bit) {
 		HA_RWLOCK_WRLOCK(OTHER_LOCK, &fd_mig_lock);
 		if (fdtab[fd].owner == expected_owner) {
 			fdtab[fd].thread_mask = tid_bit;
@@ -389,7 +390,7 @@ int fd_takeover(int fd, void *expected_owner)
 
 	new_masks[0] = new_masks[1] = tid_bit;
 
-	old_masks[0] = _HA_ATOMIC_OR(&fdtab[fd].running_mask, tid_bit);
+	old_masks[0] = _HA_ATOMIC_OR_FETCH(&fdtab[fd].running_mask, tid_bit);
 	old_masks[1] = fdtab[fd].thread_mask;
 
 	/* protect ourself against a delete then an insert for the same fd,
@@ -419,10 +420,7 @@ int fd_takeover(int fd, void *expected_owner)
 
 void updt_fd_polling(const int fd)
 {
-	if ((fdtab[fd].thread_mask & all_threads_mask) == tid_bit) {
-
-		/* note: we don't have a test-and-set yet in hathreads */
-
+	if (all_threads_mask == 1UL || (fdtab[fd].thread_mask & all_threads_mask) == tid_bit) {
 		if (HA_ATOMIC_BTS(&fdtab[fd].update_mask, tid))
 			return;
 
@@ -448,7 +446,6 @@ void updt_fd_polling(const int fd)
 	}
 }
 
-__decl_spinlock(log_lock);
 /* Tries to send <npfx> parts from <prefix> followed by <nmsg> parts from <msg>
  * optionally followed by a newline if <nl> is non-null, to file descriptor
  * <fd>. The message is sent atomically using writev(). It may be truncated to
@@ -462,6 +459,7 @@ __decl_spinlock(log_lock);
 ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t npfx, const struct ist msg[], size_t nmsg, int nl)
 {
 	struct iovec iovec[32];
+	size_t totlen = 0;
 	size_t sent = 0;
 	int vec = 0;
 	int attempts = 0;
@@ -488,6 +486,7 @@ ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t
 		iovec[vec].iov_base = pfx->ptr;
 		iovec[vec].iov_len  = MIN(maxlen, pfx->len);
 		maxlen -= iovec[vec].iov_len;
+		totlen += iovec[vec].iov_len;
 		if (iovec[vec].iov_len)
 			vec++;
 		pfx++; npfx--;
@@ -505,7 +504,7 @@ ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t
 	 * up.
 	 */
 
-	while (HA_SPIN_TRYLOCK(OTHER_LOCK, &log_lock) != 0) {
+	while (HA_ATOMIC_BTS(&fdtab[fd].state, FD_EXCL_SYSCALL_BIT)) {
 		if (++attempts >= 200) {
 			/* so that the caller knows the message couldn't be delivered */
 			sent = -1;
@@ -515,13 +514,13 @@ ssize_t fd_write_frag_line(int fd, size_t maxlen, const struct ist pfx[], size_t
 		ha_thread_relax();
 	}
 
-	if (unlikely(!fdtab[fd].initialized)) {
-		fdtab[fd].initialized = 1;
+	if (unlikely(!(fdtab[fd].state & FD_INITIALIZED))) {
+		HA_ATOMIC_OR(&fdtab[fd].state, FD_INITIALIZED);
 		if (!isatty(fd))
 			fcntl(fd, F_SETFL, O_NONBLOCK);
 	}
 	sent = writev(fd, iovec, vec);
-	HA_SPIN_UNLOCK(OTHER_LOCK, &log_lock);
+	HA_ATOMIC_BTR(&fdtab[fd].state, FD_EXCL_SYSCALL_BIT);
 
  leave:
 	/* sent > 0 if the message was delivered */
@@ -675,8 +674,7 @@ static void deinit_pollers_per_thread()
 /* Release the pollers per thread, to be called late */
 static void free_pollers_per_thread()
 {
-	free(fd_updt);
-	fd_updt = NULL;
+	ha_free(&fd_updt);
 }
 
 /*
@@ -752,9 +750,9 @@ void deinit_pollers() {
 			bp->term(bp);
 	}
 
-	free(fdinfo);   fdinfo   = NULL;
-	free(fdtab_addr);    fdtab_addr    = NULL;
-	free(polled_mask); polled_mask = NULL;
+	ha_free(&fdinfo);
+	ha_free(&fdtab_addr);
+	ha_free(&polled_mask);
 }
 
 /*
@@ -824,7 +822,7 @@ int fork_poller()
 	int fd;
 	for (fd = 0; fd < global.maxsock; fd++) {
 		if (fdtab[fd].owner) {
-			fdtab[fd].cloned = 1;
+			HA_ATOMIC_OR(&fdtab[fd].state, FD_CLONED);
 		}
 	}
 
@@ -839,7 +837,7 @@ int fork_poller()
 
 /* config parser for global "tune.fd.edge-triggered", accepts "on" or "off" */
 static int cfg_parse_tune_fd_edge_triggered(char **args, int section_type, struct proxy *curpx,
-                                      struct proxy *defpx, const char *file, int line,
+                                      const struct proxy *defpx, const char *file, int line,
                                       char **err)
 {
 	if (too_many_args(1, args, err, NULL))
@@ -858,7 +856,7 @@ static int cfg_parse_tune_fd_edge_triggered(char **args, int section_type, struc
 
 /* config keyword parsers */
 static struct cfg_kw_list cfg_kws = {ILH, {
-	{ CFG_GLOBAL, "tune.fd.edge-triggered", cfg_parse_tune_fd_edge_triggered },
+	{ CFG_GLOBAL, "tune.fd.edge-triggered", cfg_parse_tune_fd_edge_triggered, KWF_EXPERIMENTAL },
 	{ 0, NULL, NULL }
 }};
 
