@@ -42,6 +42,7 @@ static int mem_fail_rate = 0;
 
 #if defined(HA_HAVE_MALLOC_TRIM)
 static int using_libc_allocator = 0;
+static int disable_trim __read_mostly = 0;
 
 /* ask the allocator to trim memory pools.
  * This must run under thread isolation so that competing threads trying to
@@ -53,6 +54,9 @@ static void trim_all_pools(void)
 {
 	int isolated = thread_isolated();
 
+	if (disable_trim)
+		return;
+
 	if (!isolated)
 		thread_isolate();
 
@@ -163,11 +167,16 @@ struct pool_head *create_pool(char *name, unsigned int size, unsigned int flags)
 	}
 
 	if (!pool) {
-		if (!pool)
-			pool = calloc(1, sizeof(*pool));
+		void *pool_addr;
 
-		if (!pool)
+		pool_addr = calloc(1, sizeof(*pool) + __alignof__(*pool));
+		if (!pool_addr)
 			return NULL;
+
+		/* always provide an aligned pool */
+		pool = (struct pool_head*)((((size_t)pool_addr) + __alignof__(*pool)) & -(size_t)__alignof__(*pool));
+		pool->base_addr = pool_addr; // keep it, it's the address to free later
+
 		if (name)
 			strlcpy2(pool->name, name, sizeof(pool->name));
 		pool->size = size;
@@ -491,7 +500,7 @@ void *pool_destroy(struct pool_head *pool)
 			HA_SPIN_DESTROY(&pool->lock);
 #endif
 			/* note that if used == 0, the cache is empty */
-			free(pool);
+			free(pool->base_addr);
 		}
 	}
 	return NULL;
@@ -662,10 +671,26 @@ static int mem_parse_global_fail_alloc(char **args, int section_type, struct pro
 }
 #endif
 
+#if defined(HA_HAVE_MALLOC_TRIM)
+/* config parser for global "no-memory-trimming" */
+static int mem_parse_global_no_mem_trim(char **args, int section_type, struct proxy *curpx,
+                                       const struct proxy *defpx, const char *file, int line,
+                                       char **err)
+{
+	if (too_many_args(0, args, err, NULL))
+		return -1;
+	disable_trim = 1;
+	return 0;
+}
+#endif
+
 /* register global config keywords */
 static struct cfg_kw_list mem_cfg_kws = {ILH, {
 #ifdef DEBUG_FAIL_ALLOC
 	{ CFG_GLOBAL, "tune.fail-alloc", mem_parse_global_fail_alloc },
+#endif
+#if defined(HA_HAVE_MALLOC_TRIM)
+	{ CFG_GLOBAL, "no-memory-trimming", mem_parse_global_no_mem_trim },
 #endif
 	{ 0, NULL, NULL }
 }};
